export { Interface as Interface2, InterfaceWithFields, NonDefaultInterface, Type, Bar, ReExportedName as NewReExportedName } from './re-export';
export { NonDefaultInterface as Foo } from 'package-with-default-export';
