export interface CInterfaceName {}

export interface BInterfaceName {
	prop: number;
	prop2: ATypeName;
}

export type BType = number;
export type ATypeName = string | number;

/**
 * Some description
 */
export type ZTypeName = string | null;
