import MyInterface from './interface';
import MyClass from './class';
import MyAnotherClass from './another-class';

export default class MyNewClass extends MyClass implements MyInterface {}

export class MyNewClass2 extends MyAnotherClass { }
