import { default as myClass5 } from 'package-with-default-export';

declare function _default(first: number): void;
declare function _default$1(second: number): void;
declare class _default$2 {
	first: number;
}
declare class _default$3 {
	second: number;
}
declare const _default$4: 0;
declare const _default$5: "";
declare const _default$6: {
	type: string;
};

export {
	_default as myFunc1,
	_default as myFunc2,
	_default$1 as myFunc3,
	_default$1 as myFunc4,
	_default$2 as myClass1,
	_default$2 as myClass2,
	_default$3 as myClass3,
	_default$3 as myClass4,
	_default$4 as number,
	_default$5 as string,
	_default$6 as object,
	myClass5,
};

export {};
