export { default as myFunc1 } from './func';
export { default as myFunc2 } from './func';
export { default as myFunc3 } from './another-func';
export { default as myFunc4 } from './another-func';

export { default as myClass1 } from './class';
export { default as myClass2 } from './class';
export { default as myClass3 } from './another-class';
export { default as myClass4 } from './another-class';

export { default as number } from './number';
export { default as string } from './string';
export { default as object } from './object';

export { default as myClass5 } from 'package-with-default-export';
