export interface BInterfaceName {
	prop: number;
	prop2: ATypeName;
}
export interface CInterfaceName {
}
export type ATypeName = string | number;
export type BType = number;
/**
 * Some description
 */
export type ZTypeName = string | null;

export {};
