declare namespace ora {
	interface Options {
	}
}
declare const ora: {
	(options: ora.Options): any;
};
export declare const ExportedValue: ora.Options;
export type ExportedType = ora.Options;

export {};
