import { variable, SampleClass, ISample, SampleNS, functionName } from './library';

export namespace Sample {
	export { functionName };
	export { variable };
	export { SampleClass };
	export { ISample };
	export { SampleNS };
}
