import { InterfaceFromTypesPackage } from 'fake-types-lib';
import * as namespacedImportTypes from 'package-with-namespaced-re-export';

export interface Interface {}

export type Type = number | string;

export interface InterfaceWithFields {
	field: Type;
	field2: Interface;
	field3: InterfaceFromTypesPackage;
}

export module ModuleWithoutQuotes {
	export type A = string;
}

export { namespacedImportTypes };
