import { mergeName } from 'plugins-package';

declare module 'plugins-package' {
	namespace mergeName {
		type A = string;
	}
}

declare module 'plugins-package/foobar' {
	export function foobar(val: mergeName.OtherType): void;
}
