export type MyString = string;
export interface MyInt {}
export function func() {}

export { Interface } from 'fake-package';

export * from './type';
export * from './type2';
export * as subNs from './type2';
