import { InterfaceWithFields as FakePackageInterface } from 'fake-package';

export const myVar: FakePackageInterface = {
	field: 2,
	field2: {},
	field3: {
		field: '',
		field2: 0,
		field3: 0,
	},
};
