export namespace InternalNamespace {
	export type NSType = string;
}

export module InternalModule {
	export type MType = number;
}

export namespace ExportedNamespace {
	export type ENSType = 1 | 2;
}

export module ExportedModule {
	export type EMType = 3 | 4;
}
