declare module 'fake-types-lib-2' {
	export interface Data {
		field: number;
		method(): void;
	}

	export interface Base {
		id: string;
	}

	export interface Derived extends Base {
		name: string;
	}

	export function getData(): Data;
}

declare module 'fake-types-lib-2.5' {
	export interface Data {
		field: number;
	}
}
