/// <reference types="node" />

import { InterfaceFromTypesPackage } from 'fake-types-lib';
import { Stats } from 'fs';

export interface InterfaceName {
	prop: Stats;
	field: InterfaceFromTypesPackage;
}

export {};
