export class Class {}
export const enum ConstEnum {}
export const variable: string = '123';
export enum Enum {}
export function func(): void {}
export interface Interface {}
export type Type = string;
