declare namespace commander {
	interface Option {}
	interface OptionConstructor {
		new(): Option;
	}

	interface Command {}
	interface CommandConstructor {
		new (): Command;
	}

	interface CommanderStatic extends Command {
		Command: CommandConstructor;
		Option: OptionConstructor;
	}
}

declare const commander: commander.CommanderStatic;
export = commander;
